package dev.pfaff.altargui.glfw;

import dev.pfaff.altargui.host.ContextForker;
import dev.pfaff.altargui.host.ForkedContext;
import dev.pfaff.altargui.host.Host;
import dev.pfaff.altargui.host.IWindow;
import dev.pfaff.altargui.natives.Natives;
import dev.pfaff.altargui.util.BasicFuture;
import dev.pfaff.altargui.util.GLThread;
import dev.pfaff.altargui.util.listenable.ConstantValue;
import dev.pfaff.altargui.util.listenable.Value;
import dev.pfaff.glass.glfw.GLFWwindow;
import dev.pfaff.glass.opengl.GL;
import org.jspecify.annotations.NonNull;
import org.jspecify.annotations.Nullable;

import java.lang.foreign.SymbolLookup;

import static dev.pfaff.altargui.glfw.GLGlobals.GLFW;
import static dev.pfaff.altargui.util.BasicFuture.executeOn;

final class HostImpl implements Host {
	private static final ConstantValue<Float> CONSTANT_VALUE_1F = new ConstantValue<>(1f);

	private static final IWindow WINDOW = new IWindow() {
		@Override
		public @NonNull Value<@NonNull Float> scaleFactorListenable() {
			return CONSTANT_VALUE_1F;
		}
	};

	private final Application application;

	public HostImpl(Application application) {
		this.application = application;
	}

	@Override
	public @NonNull GL gl() {
		return GLGlobals.GL;
	}

	@Override
	public @NonNull IWindow window() {
		return WINDOW;
	}

	@Override
	// this nullability warning is wrong
	public @Nullable String getClipboardString() {
		return executeOn(this.application.window::getClipboardString, this.application::submitToMainThreadOrRun);
	}

	@Override
	public void setClipboardString(@NonNull String string) {
		executeOn(() -> {
					this.application.window.setClipboardString(string);
					return null;
				},
				this.application::submitToMainThreadOrRun);
	}

	@Override
	public @NonNull SymbolLookup loadLibImage() {
		return Natives.loadLibImage();
	}

	@Override
	public ContextForker contextForker() {
		return () -> {
			var window = BasicFuture.executeOn(() -> {
				return GLFWwindow.create(GLFW, 1, 1, "", null, this.application.window);
			}, GLThread::submitToGLThread);
			window.makeContextCurrent();
			return new ForkedContext() {
				@Override
				public GL gl() {
					return GLGlobals.GL;
				}

				@Override
				public void close() {
					window.destroy();
				}
			};
		};
	}
}
